@@ -1,9 +1,9 @@
 /*****************************************************************************
- * RRDtool 1.4.6  Copyright by Tobi Oetiker, 1997-2012
+ * RRDtool 1.4.9  Copyright by Tobi Oetiker, 1997-2014
  *****************************************************************************
  * rrd_dump  Display a RRD
  *****************************************************************************
- * $Id: rrd_dump.c 2255 2012-01-16 14:40:06Z oetiker $
+ * $Id$
  * $Log$
  * Revision 1.7  2004/05/25 20:53:21  oetiker
  * prevent small leak when resources are exhausted -- Mike Slifcak
@@ -103,9 +103,7 @@ int rrd_dump_cb_r(
         return (-1);
     }
 
-    old_locale = setlocale(LC_NUMERIC, NULL);
-    setlocale(LC_NUMERIC, "C");
-
+    old_locale = setlocale(LC_NUMERIC, "C");
 
     if (opt_header == 1) {
         CB_PUTS("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
