@@ -1,5 +1,5 @@
 /****************************************************************************
- * RRDtool 1.4.6  Copyright by Tobi Oetiker, 1997-2012
+ * RRDtool 1.4.9  Copyright by Tobi Oetiker, 1997-2014
  ****************************************************************************
  * rrd_rpncalc.c  RPN calculator functions
  ****************************************************************************/
@@ -46,7 +46,7 @@ short rpn_compact(
             /* rpnp.val is a double, rpnc.val is a short */
             double    temp = floor(rpnp[i].val);
 
-            if (temp < SHRT_MIN || temp > SHRT_MAX) {
+			if (temp < SHRT_MIN || temp > SHRT_MAX || temp != rpnp[i].val) {
                 rrd_set_error
                     ("constants must be integers in the interval (%d, %d)",
                      SHRT_MIN, SHRT_MAX);
@@ -185,6 +185,8 @@ void rpn_compact2str(
             add_op(OP_AVG, AVG)
             add_op(OP_ABS, ABS)
             add_op(OP_ADDNAN, ADDNAN)
+            add_op(OP_MINNAN, MINNAN)
+            add_op(OP_MAXNAN, MAXNAN)
 #undef add_op
     }
     (*str)[offset] = '\0';
@@ -299,8 +301,7 @@ rpnp_t   *rpn_parse(
     char      vname[MAX_VNAME_LEN + 10];
     char     *old_locale;
 
-    old_locale = setlocale(LC_NUMERIC, NULL);
-    setlocale(LC_NUMERIC, "C");
+    old_locale = setlocale(LC_NUMERIC, "C");
 
     rpnp = NULL;
     expr = (char *) expr_const;
@@ -331,6 +332,7 @@ rpnp_t   *rpn_parse(
              rpnp[steps].op = VV; \
              rpnp[steps].ptr = (*lookup)(key_hash,vname); \
              if (rpnp[steps].ptr < 0) { \
+                           rrd_set_error("variable '%s' not found",vname);\
 			   free(rpnp); \
 			   return NULL; \
 			 } else expr+=length; \
@@ -387,6 +389,8 @@ rpnp_t   *rpn_parse(
             match_op(OP_AVG, AVG)
             match_op(OP_ABS, ABS)
             match_op(OP_ADDNAN, ADDNAN)
+            match_op(OP_MINNAN, MINNAN)
+            match_op(OP_MAXNAN, MAXNAN)
 #undef match_op
             else if ((sscanf(expr, DEF_NAM_FMT "%n", vname, &pos) == 1)
                      && ((rpnp[steps].ptr = (*lookup) (key_hash, vname)) !=
@@ -396,6 +400,7 @@ rpnp_t   *rpn_parse(
         }
 
         else {
+            rrd_set_error("don't undestand '%s'",expr);
             setlocale(LC_NUMERIC, old_locale);
             free(rpnp);
             return NULL;
@@ -733,6 +738,15 @@ short rpn_calc(
                 rpnstack->s[stptr - 1] = rpnstack->s[stptr];
             stptr--;
             break;
+        case OP_MINNAN:
+            stackunderflow(1);
+            if (isnan(rpnstack->s[stptr - 1]))
+                rpnstack->s[stptr - 1] = rpnstack->s[stptr];
+            else if (isnan(rpnstack->s[stptr]));
+            else if (rpnstack->s[stptr - 1] > rpnstack->s[stptr])
+                rpnstack->s[stptr - 1] = rpnstack->s[stptr];
+            stptr--;
+            break;
         case OP_MAX:
             stackunderflow(1);
             if (isnan(rpnstack->s[stptr - 1]));
@@ -742,6 +756,15 @@ short rpn_calc(
                 rpnstack->s[stptr - 1] = rpnstack->s[stptr];
             stptr--;
             break;
+        case OP_MAXNAN:
+            stackunderflow(1);
+            if (isnan(rpnstack->s[stptr - 1]))
+                rpnstack->s[stptr - 1] = rpnstack->s[stptr];
+            else if (isnan(rpnstack->s[stptr]));
+            else if (rpnstack->s[stptr - 1] < rpnstack->s[stptr])
+                rpnstack->s[stptr - 1] = rpnstack->s[stptr];
+            stptr--;
+            break;
         case OP_LIMIT:
             stackunderflow(2);
             if (isnan(rpnstack->s[stptr - 2]));
@@ -886,7 +909,9 @@ short rpn_calc(
                 if (output_idx + 1 >= (int) ceil((float) dur / (float) step)) {
                     int       ignorenan = (rpnp[rpi].op == OP_TREND);
                     double    accum = 0.0;
-                    int       i = 0;
+                    int       i = -1; /* pick the current entries, not the next one
+                                         as the data pointer has already been forwarded
+                                         when the OP_VARIABLE was processed */
                     int       count = 0;
 
                     do {
